////////////////////////////////////////////////////////////
// DAB Shield Example App
// AVIT Research Ltd
// v0.1 11/05/2017 - initial release
// v0.2 09/10/2017 - support for M0 etc.
// v0.3 17/10/2017 - Added Max Services to prevent overflow (Library only)
// v0.4 20/11/2017 - Added Version Info
// v0.5 23/11/2017 - Added More 'Getting Started' Debugging
// v0.6 23/11/2017 - support for Due etc.
// v0.7 27/11/2017 - Added Get Time support
// v0.8 06/12/2017 - Added FM/RDS Functionality
// v0.9 05/09/2019 - Enhanced FM functionality (seek/scan)
// v1.0 23/09/2020 - Added ESP32 D1 R32 Support
// v1.1 11/12/2020 - Added DAB Status, Mono and Mute
// v1.2 01/11/2021 - Added DAB Service Type, Dab/Dab+
///////////////////////////////////////////////////////////
// DABUartCtrl: DAB tuner controlled by uart
// Dr.-Ing. Ingo D. Rullhusen <d01c@freeshell.de>
// v0.1 16/11/2024 - start
// v0.2 03/12/2024 - first full function
// v0.3 31/12/2024 - final
///////////////////////////////////////////////////////////
#include <SPI.h>
#include <DABShield.h>
//In order to compile for the UNO, all strings are put into flash (PROGRAM MEMORY)
const char pty_0[] PROGMEM =  "None";
const char pty_1[] PROGMEM =  "News";
const char pty_2[] PROGMEM =  "Current affairs";
const char pty_3[] PROGMEM =  "Information";
const char pty_4[] PROGMEM =  "Sport";
const char pty_5[] PROGMEM =  "Education";
const char pty_6[] PROGMEM =  "Drama";
const char pty_7[] PROGMEM =  "Culture";
const char pty_8[] PROGMEM =  "Science";
const char pty_9[] PROGMEM =  "Varied";
const char pty_10[] PROGMEM =  "Pop music";
const char pty_11[] PROGMEM =  "Rock music";
const char pty_12[] PROGMEM =  "Easy listening music";
const char pty_13[] PROGMEM =  "Light classical";
const char pty_14[] PROGMEM =  "Serious classical";
const char pty_15[] PROGMEM =  "Other music";
const char pty_16[] PROGMEM =  "Weather";
const char pty_17[] PROGMEM =  "Finance";
const char pty_18[] PROGMEM =  "Children’s programmes";
const char pty_19[] PROGMEM =  "Social Affairs";
const char pty_20[] PROGMEM =  "Religion";
const char pty_21[] PROGMEM =  "Phone In";
const char pty_22[] PROGMEM =  "Travel";
const char pty_23[] PROGMEM =  "Leisure";
const char pty_24[] PROGMEM =  "Jazz music";
const char pty_25[] PROGMEM =  "Country music";
const char pty_26[] PROGMEM =  "National music";
const char pty_27[] PROGMEM =  "Oldies music";
const char pty_28[] PROGMEM =  "Folk music";
const char pty_29[] PROGMEM =  "Documentary";
const char pty_30[] PROGMEM =  "Alarm test";
const char pty_31[] PROGMEM =  "Alarm";
const char *const pty[] PROGMEM = {pty_0,pty_1,pty_2,pty_3,pty_4,pty_5,pty_6,pty_7,pty_8,pty_9,pty_10,pty_11,pty_12,pty_13,pty_14,pty_15,pty_16,pty_17,pty_18,pty_19,pty_20,pty_21,pty_22,pty_23,pty_24,pty_25,pty_26,pty_27,pty_28,pty_29,pty_30,pty_31};

const char mode_0[] PROGMEM = "Dual";
const char mode_1[] PROGMEM = "Mono";
const char mode_2[] PROGMEM = "Stereo";
const char mode_3[] PROGMEM = "Joint Stereo";
const char *const audiomode[] PROGMEM = {mode_0,mode_1,mode_2,mode_3};

#define DAB_SPI_BITBANG

#ifdef ARDUINO_ARCH_SAMD
#define Serial SerialUSB
//Comment out the following line if using the ICSP connector (and modified DABShield).
#define DAB_SPI_BITBANG
#endif

#ifdef ARDUINO_ARCH_SAM
#define Serial SerialUSB
//Comment out the following line if using the ICSP connector (and modified DABShield).
#define DAB_SPI_BITBANG
#endif

DAB Dab;
//SPI Ports for BIT
#if defined(ARDUINO_ARCH_ESP32)
const byte slaveSelectPin = 12;
#else
const byte slaveSelectPin = 8;
#endif

#ifdef DAB_SPI_BITBANG
#if defined(ARDUINO_ARCH_ESP32)
const byte SCKPin = 18;
const byte MISOPin = 19;
const byte MOSIPin = 23;
#else
const byte SCKPin = 13;
const byte MISOPin = 12;
const byte MOSIPin = 11;
#endif
#endif

DABTime dabtime;
uint8_t vol = 63;
uint8_t service = 0;
uint8_t freq = 0;

byte rxindex = 0;
char rxdata[32];

extern byte DABResetPin;


// if used: USB console must be present at start
// if  not: load by pressing board reset or using programmer
//#define USB_CDC_ENABLED

const byte DABCTLPin = 0;

const byte STEREOPin = 5;
const byte MONOPin = 4;
const byte CHNTUNPin = 9;
const byte STNTUNPin = 10;

const byte RSSIPin = 3;

bool mono_switch = false;
bool stn_selected = false;

void setup() {

#ifdef USB_CDC_ENABLED
  //Intitialise the terminal
  Serial.begin(115200);
  while(!Serial);

  Serial.print(F("                    AVIT&IDR DABUartCtrl 2024\n\n")); 
#endif
  
  //Enable SPI
#ifdef DAB_SPI_BITBANG
  pinMode(slaveSelectPin, OUTPUT);
  pinMode(SCKPin, OUTPUT);
  pinMode(MOSIPin, OUTPUT);
  pinMode(MISOPin, INPUT_PULLUP);
  digitalWrite(slaveSelectPin, HIGH);
#else 
  pinMode(slaveSelectPin, OUTPUT);
  digitalWrite(slaveSelectPin, HIGH);
  SPI.begin();
#endif

   digitalWrite(STEREOPin, LOW);
   pinMode(STEREOPin, OUTPUT);
   pinMode(MONOPin, INPUT_PULLUP);
   digitalWrite(CHNTUNPin, LOW);
   pinMode(CHNTUNPin, OUTPUT);
   digitalWrite(STNTUNPin, LOW);
   pinMode(STNTUNPin, OUTPUT);

   pinMode( DABCTLPin, INPUT_PULLUP );
   Serial1.begin( 9600, SERIAL_8O1 );
   while( !Serial1 );
   while( Serial1.available() > 0 ) { Serial1.read(); }

   analogWrite( RSSIPin, 0 );
   pinMode( RSSIPin, OUTPUT );

#ifdef USB_CDC_ENABLED
  Serial.print(F("Initialising...<wait for timeout>...")); 
#endif

  //DAB Setup
  Dab.setCallback(ServiceData);
  Dab.begin();
  Dab.vol(vol);
  Dab.tune(freq);

#ifdef USB_CDC_ENABLED
  if(Dab.error != 0)
  {
    Serial.print(F("ERROR: "));
    Serial.print(Dab.error);
    Serial.print(F("\nCheck DABShield is Connected and SPI Communications\n"));
  }
  else  
  {
    Serial.print(F("done\n\n"));
    Help_Menu();
  }
  Serial.print(F("DAB>"));
#endif

   digitalWrite( CHNTUNPin, HIGH );
}

void ServiceData(void)
{
/***
    Serial.print(Dab.ServiceData);
    Serial.print(F("\n"));
***/
}

void loop() {
  // put your main code here, to run repeatedly:

  Dab.task();
#ifdef USB_CDC_ENABLED
  if (Serial.available() > 0)
  {
    rxdata[rxindex] = Serial.read();
    if (rxdata[rxindex] == '\r' || rxdata[rxindex] == '\n')  //return or linefeed
    {
      if ( rxdata[rxindex] == '\r' && Serial.peek() == '\n' ) { Serial.read(); }
      if ( rxdata[rxindex] == '\n' && Serial.peek() == '\r' ) { Serial.read(); }
      Serial.println();
      rxdata[rxindex] = '\0';

      process_command(rxdata);

      rxindex = 0;
    }
    else if (rxdata[rxindex] == '\b' || rxdata[rxindex] == 127 )  //backspace or DEL
    {
      if (rxindex > 0)
      {
        Serial.print(F("\b \b"));
        rxindex--;
      }
    }
    else if (rxdata[rxindex] == '+')
    {
      if (vol < 63)
      {
        vol++;
        Dab.vol(vol);
      }
    }
    else if (rxdata[rxindex] == '-')
    {
      if (vol > 0)
      {
        vol--;
        Dab.vol(vol);
      }
    }
    else if (rxdata[rxindex] == '<')
    {
      if (Dab.numberofservices > 0)
      {
        if (service > 0)
        {
          service--;
        }
        else
        {
          service = Dab.numberofservices - 1;
        }
        Dab.set_service(service);
        Serial.print(Dab.service[service].Label);
        Serial.print(F("\n"));
      }
    }
    else if (rxdata[rxindex] == '>')
    {
      if (Dab.numberofservices > 0)
      {
        if (service < (Dab.numberofservices - 1))
        {
          service++;
        }
        else
        {
          service = 0;
        }
        Dab.set_service(service);
        Serial.print(Dab.service[service].Label);
        Serial.print(F("\n"));
      }
    }
    else  //other char
    {
      Serial.print(rxdata[rxindex]);
      rxindex++;
      if (rxindex >= 32)
      {
        rxindex = 0;
      }
    }
  }
#endif
   
   if ( Serial1.available() > 0 )
   {
      int cmdByte = Serial1.read();
      if ( cmdByte == 255 )
      {
#ifdef USB_CDC_ENABLED
         Serial.println( F("\nTERMINATE.") );
#endif
         Dab.mute( true, true );
         stn_selected = false;
         digitalWrite( STEREOPin, LOW );
         digitalWrite( STNTUNPin, LOW );
         digitalWrite( CHNTUNPin, LOW );
         digitalWrite( DABResetPin, LOW );
         analogWrite( RSSIPin, 0 );
         delay( 100 );
         exit( 1 );
      }

      if ( cmdByte > 127 )
      {
         int i;
#ifdef USB_CDC_ENABLED
         Serial.print( F("\nSTNTUNE ") );
#endif
         Dab.mute( true, true );
         stn_selected = false;
         digitalWrite( STEREOPin, LOW );
         digitalWrite( STNTUNPin, LOW );
#ifdef USB_CDC_ENABLED
         Serial.print( 0x3f & cmdByte );
         Serial.print( F(" -> ") );
#endif
         for ( i = 0; i < Dab.numberofservices; i++ )
         {
            if ( ( (Dab.service[i].CompID & 0x8000) == 0 ) && ( ( (Dab.service[i].CompID ^ cmdByte) & 0x3f ) == 0 ) )
            {
               service = i;
#ifdef USB_CDC_ENABLED
               Serial.print( service );
#endif
               Dab.set_service( service );
               if ( Dab.servicevalid() )
               {
                  stn_selected = true;
                  Dab.mute( false, false );
               }
            }
         }
#ifdef USB_CDC_ENABLED
         Serial.print( F("\nDAB>") );
#endif
      }
      else
      {
#ifdef USB_CDC_ENABLED
         Serial.print( F("\nCHNTUNE ") );
#endif
         Dab.mute( true, true );
         stn_selected = false;
         digitalWrite( STEREOPin, LOW );
         digitalWrite( STNTUNPin, LOW );
         digitalWrite( CHNTUNPin, LOW );
         freq = 0x3f & cmdByte;
         service = 0;
         Dab.tune(freq);
#ifdef USB_CDC_ENABLED
         Serial.print( freq );
         Serial.print( F(" ") );
         Serial.print( Dab.servicevalid() );
         Serial.print( F("\nDAB>") );
#endif
      }

   }

   if ( mono_switch != digitalRead( MONOPin ) )
   {
      mono_switch = digitalRead( MONOPin );
      if ( mono_switch )
      {
         Dab.mono(true);
      }
      else
      {
         Dab.mono(false);
      }
   }

   Dab.status();
   digitalWrite( CHNTUNPin, ( Dab.quality > 0 ) && ( Dab.quality <= 100 ) );
   digitalWrite( STNTUNPin, stn_selected && ( Dab.quality > 80 && Dab.quality <= 100 ) );
   digitalWrite( STEREOPin, stn_selected && ( Dab.quality > 0 ) && ( Dab.mode != MONO ) && (! mono_switch ) );

   // data sheet: -128...63
   // observed: 12...63 -> 0...3V
   // measured: 0.6V per LED, 0.2V injection diode drop
   float rssi = 255*(Dab.signalstrength-24) / (63-24);
   if ( rssi <   0 ) { rssi =   0; }
   if ( rssi > 255 ) { rssi = 255; }
   analogWrite( RSSIPin, 0.6*rssi + 255*(0.2/5.0) );

}

#ifdef USB_CDC_ENABLED

void Help_Menu(void)
{
  Serial.print(F("                        HELP\n")); 
  Serial.print(F("PartNo = Si"));
  Serial.print(Dab.PartNo);
  Serial.print(F("\n"));
  
  Serial.print(F("Firmware Version = "));
  Serial.print(Dab.VerMajor);
  Serial.print(F("."));
  Serial.print(Dab.VerMinor);
  Serial.print(F("."));
  Serial.print(Dab.VerBuild);
  Serial.print(F("\n"));
  
  Serial.print(F("________________________________________________________\n\n"));
    Serial.print(F("scan                     - scans for valid DAB ensembles\n"));
    Serial.print(F("tune <n>                 - tunes to frequency index\n"));
    Serial.print(F("service <ID>             - tunes to service ID\n"));
    Serial.print(F("info                     - displays the ensemble info\n"));
    Serial.print(F("time                     - displays the current DAB time\n"));
  Serial.print(F("volume <n>               - set volume 0 - 63\n"));
  Serial.print(F("mono                     - set audio mode to mono\n"));
  Serial.print(F("stereo                   - set audio mode to stereo\n"));
  Serial.print(F("mute <on/off/left/right> - mutes/unmutes audio\n"));
  Serial.print(F("status                   - displays audio/reception info\n"));
  Serial.print(F("help                     - displays this menu\n"));
  Serial.print(F("________________________________________________________\n\n"));
}

void process_command(char *command)
{
  char *cmd;
  cmd = strtok(command, " \r");
  if (strcmp_P(cmd, PSTR("tune")) == 0)
  {
    cmd = strtok(NULL, " \r");

      freq = (int)strtol(cmd, NULL, 10);
      service = 0;
      Dab.tune(freq);
      Ensemble_Info();

  }
  else if (strcmp_P(cmd, PSTR("service")) == 0)
  {
    cmd = strtok(NULL, " \r");
    service = (uint8_t)strtol(cmd, NULL, 10);
    Dab.set_service(service);
    Serial.print(Dab.service[service].Label);
    Serial.print(F("\n"));
  }
  else if (strcmp_P(cmd, PSTR("volume")) == 0)
  {
    cmd = strtok(NULL, " \r");
    vol = (uint8_t)strtol(cmd, NULL, 10);
    Dab.vol(vol);
  }
  else if (strcmp_P(cmd, PSTR("info")) == 0)
  {
    Ensemble_Info();
  }
  else if (strcmp_P(cmd, PSTR("scan")) == 0)
  {
    DAB_scan();
  }
  else if (strcmp(cmd, "mono") == 0)
  {
    Dab.mono(true);
  }
  else if (strcmp(cmd, "stereo") == 0)
  {
    Dab.mono(false);
  }
  else if (strcmp(cmd, "mute") == 0)
  {
    cmd = strtok(NULL, " \r");
    if(strcmp(cmd, "on") == 0)
    {
      Dab.mute(true, true);  
    }
    else if(strcmp(cmd, "off") == 0)
    {
      Dab.mute(false, false);  
    }    
    else if(strcmp(cmd, "left") == 0)
    {
      Dab.mute(true, false);  
    }    
    else if(strcmp(cmd, "right") == 0)
    {
      Dab.mute(false, true);  
    }    
  }  
  else if (strcmp(cmd, "status") == 0)
  {
    DAB_status();
  }
  else if (strcmp(cmd, "time") == 0)
  {
    char timestring[16];
    Dab.time(&dabtime);
    sprintf(timestring,"%02d/%02d/%02d ", dabtime.Days,dabtime.Months,dabtime.Year);
    Serial.print(timestring);
    sprintf(timestring,"%02d:%02d\n", dabtime.Hours,dabtime.Minutes);
    Serial.print(timestring);
  } 
  else if (strcmp(cmd, "help") == 0)
  {
    Help_Menu();
  }
  else if (strlen(command) == 0)
  {
    //no command
  }
  else
  {
    Serial.print(F("Unknown command\n"));
  }

  Serial.print(F("DAB>"));

}

void Ensemble_Info(void)
{
  char freqstring[32];
  uint8_t i;

  Serial.print(F("\n\nEnsemble Freq "));
  sprintf(freqstring, "%02d\t %03d.", Dab.freq_index, (uint16_t)(Dab.freq_khz(Dab.freq_index) / 1000));
  Serial.print(freqstring);
  sprintf(freqstring, "%03d MHz", (uint16_t)(Dab.freq_khz(Dab.freq_index) % 1000));
  Serial.println(freqstring);

  Serial.print(Dab.EnsembleID);
  Serial.print(F(":\t"));
  Serial.println(Dab.Ensemble);

  Serial.print(F("\nServices: \n"));
  Serial.print(F("ID\tSubCh\tName             Type\n\n"));

  for (i = 0; i < Dab.numberofservices; i++)
  {
    Serial.print(i);
    Serial.print(F(":\t"));
    if ( Dab.service[i].CompID & 0x8000 ) { Serial.print( F("-") );                     }
    else                                  { Serial.print( Dab.service[i].CompID&0x3f ); }
    Serial.print(F("\t"));
    Serial.print(Dab.service[i].Label);
    Dab.status(Dab.service[i].ServiceID, Dab.service[i].CompID);
    if(Dab.type == SERVICE_AUDIO)
    {
      Serial.print(F(" dab"));
      if(Dab.dabplus == true)
      {
        Serial.print(F("+"));
      }
    }
    else if(Dab.type == SERVICE_DATA)
    {
      Serial.print(F(" data"));
    }
    Serial.print(F("\n"));
  }
  Serial.print(F("\n"));
}

void DAB_scan(void)
{
  uint8_t freq_index;
  char freqstring[32];

  for (freq_index = 0; freq_index < DAB_FREQS; freq_index++)
  {
    Serial.print(F("\nScanning Freq "));
    sprintf(freqstring, "%02d\t %03d.", freq_index, (uint16_t)(Dab.freq_khz(freq_index) / 1000));
    Serial.print(freqstring);
    sprintf(freqstring, "%03d MHz", (uint16_t)(Dab.freq_khz(freq_index) % 1000));
    Serial.print(freqstring);
    Dab.tune(freq_index);
    if(Dab.servicevalid() == true)
    {
      Ensemble_Info();
    }
  }
  Serial.print(F("\n\n"));
}

void DAB_status(void)
{
  char dabstring[32];
  Dab.status();
  Serial.println(Dab.service[service].Label);
  sprintf_P(dabstring,PSTR("PTY = %S (%d)"), pgm_read_word(&pty[Dab.pty]), Dab.pty);
  Serial.println(dabstring);

  sprintf_P(dabstring,PSTR("Bit Rate = %u kHz, "), Dab.bitrate);
  Serial.print(dabstring);
  sprintf_P(dabstring,PSTR("Sample Rate = %u Hz, "), Dab.samplerate);
  Serial.print(dabstring); 
  sprintf_P(dabstring,PSTR("Audio Mode = %S (%d)"), pgm_read_word(&audiomode[Dab.mode]), Dab.mode);
  Serial.println(dabstring);

  sprintf_P(dabstring, PSTR("Service Mode = %S"), (Dab.dabplus == true ? PSTR("dab+") : PSTR("dab")) );
  Serial.println(dabstring);
  
  sprintf_P(dabstring,PSTR("RSSI = %d, SNR = %d, Quality = %u%, "), Dab.signalstrength, Dab.snr, Dab.quality);
  Serial.println(dabstring);
}

#else

// no USB

int main()
{
   init();
   USBCON = 0b00100000;
   setup();
   while (true)
   {
      loop();
   }
   return(0);
}

#endif

#ifdef DAB_SPI_BITBANG
void DABSpiMsg(unsigned char *data, uint32_t len)
{
  digitalWrite(SCKPin, LOW);
  digitalWrite (slaveSelectPin, LOW);
  for(uint32_t l=0; l<len; l++)
  {
    unsigned char spiByte = data[l];
    for (uint32_t i = 0; i < 8; i++)
    {
      digitalWrite(MOSIPin, (spiByte & 0x80) ? HIGH : LOW);
      delayMicroseconds(1);
      digitalWrite(SCKPin, HIGH);
      spiByte = (spiByte << 1) | digitalRead(MISOPin);
      digitalWrite(SCKPin, LOW);
      delayMicroseconds(1);
    }
    data[l] = spiByte;
  }
  digitalWrite (slaveSelectPin, HIGH);
}
#else
void DABSpiMsg(unsigned char *data, uint32_t len)
{
  SPI.beginTransaction(SPISettings(2000000, MSBFIRST, SPI_MODE0));    //2MHz for starters...
  digitalWrite (slaveSelectPin, LOW);
  SPI.transfer(data, len);
  digitalWrite (slaveSelectPin, HIGH);
  SPI.endTransaction();
}
#endif
