//
// Send a byte per uart bitbanging
// (c) 25.12.2024 Ingo D. Rullhusen with help from ChatGPT
//

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define BAUDRATE 9600
#define BIT_TIME ( (1000000 / BAUDRATE) - 8 ) // tweaked
#define TX_PIN PD7

void uart_bitbang_init(void)
{
// Set TX_PIN (PD7) as output
   DDRD |= (1 << TX_PIN);
// Set TX_PIN high (idle state)
   PORTD |= (1 << TX_PIN);
}

void uart_bitbang_send_byte( uint8_t byte )
{
   uint8_t intr_state;
   uint8_t parity = 1; // Start mit Parität = 1 (ungerade)

   intr_state = SREG;
   cli();

// Startbit (Low)
   PORTD &= ~(1 << TX_PIN);
   _delay_us(BIT_TIME);

// 8 Datenbits senden (LSB first)
   for (uint8_t i = 0; i < 8; i++)
   {
      if (byte & (1 << i))
      {
         PORTD |= (1 << TX_PIN); // High
         parity ^= 1;            // Parität umdrehen
      }
      else
      {
         PORTD &= ~(1 << TX_PIN); // Low
      }
      _delay_us(BIT_TIME);
   }

// Paritätsbit (ungerade)
   if (parity)
   {
      PORTD |= (1 << TX_PIN);  // High
   }
   else
   {
      PORTD &= ~(1 << TX_PIN); // Low
   }
   _delay_us(BIT_TIME);

// Stoppbit (High)
   PORTD |= (1 << TX_PIN);
   _delay_us(BIT_TIME);

   SREG = intr_state;
}
