//
// Tuner functionality
// (c) 31.12.2024 Ingo D. Rullhusen
//

#ifndef TUNER_H
#define TUNER_H

#include "timer.h"

#define LED_CONFIG      (DDRD |= (1<<6))
#define LED_ON          (PORTD |= (1<<6))
#define LED_OFF         (PORTD &= ~(1<<6))
#define CPU_PRESCALE(n) (CLKPR = 0x80, CLKPR = (n))


//#define USB_CDC_ENABLED


void tuner_init(void);

void tuner_pwr_on(void);
void tuner_pwr_off(void);
uint8_t tuner_pwr_state(void);


uint8_t LOCKDET(void);

uint8_t STATDET(void);

uint8_t PWRDET(void);

uint8_t CHNTUNED(void);

uint8_t STNTUNED(void);

#define BAND_OFF 0
#define BAND_DAB 1
#define BAND_MW  2
#define BAND_FM  4
void set_band( uint8_t b );

void set_mute( uint8_t f );

void set_power( uint8_t f );


typedef int freq10k_fm;
typedef int freq1k_am;
typedef struct { int8_t channel, subchannel; } freq_dab;

extern freq10k_fm fm_lower, fm_upper;
extern freq1k_am am_lower, am_upper;
extern freq_dab dab_lower, dab_upper;

extern freq10k_fm FMstation[8];
extern freq1k_am AMstation[8];
extern freq_dab DABstation[8];

extern uint8_t last_band, last_fm_pLED, last_am_pLED, last_dab_pLED;
extern freq10k_fm last_fm_freq;
extern freq1k_am last_am_freq;
extern freq_dab last_dab_freq;

void tune_to_fm(void);
void tune_to_am(void);
void tune_to_dab( _Bool chn_changed );

void setup_to_eeprom(void);
void setup_from_eeprom(void);


void tuner_wait(void);
void tuner_on(void);


void show_message(void);


#endif
